# The labels for the output of the ner model.
# A string label can be obtained by an output index
ner_labels = ['O',
           'S-GPE',
           'S-ORG',
           'S-CARDINAL',
           'B-ORG',
           'E-ORG',
           'B-DATE',
           'E-DATE',
           'S-NORP',
           'B-GPE',
           'E-GPE',
           'S-EVENT',
           'S-DATE',
           'S-PRODUCT',
           'S-LOC',
           'I-ORG',
           'S-PERSON',
           'S-ORDINAL',
           'B-PERSON',
           'I-PERSON',
           'E-PERSON',
           'B-LAW',
           'I-LAW',
           'E-LAW',
           'B-MONEY',
           'I-MONEY',
           'E-MONEY',
           'B-EVENT',
           'I-EVENT',
           'E-EVENT',
           'B-FAC',
           'E-FAC',
           'I-DATE',
           'S-PERCENT',
           'B-QUANTITY',
           'E-QUANTITY',
           'B-WORK_OF_ART',
           'I-WORK_OF_ART',
           'E-WORK_OF_ART',
           'I-FAC',
           'S-LAW',
           'S-TIME',
           'B-LOC',
           'E-LOC',
           'I-LOC',
           'S-FAC',
           'B-TIME',
           'E-TIME',
           'S-WORK_OF_ART',
           'B-PRODUCT',
           'E-PRODUCT',
           'B-CARDINAL',
           'E-CARDINAL',
           'S-MONEY',
           'S-LANGUAGE',
           'I-TIME',
           'I-PRODUCT',
           'I-GPE',
           'I-QUANTITY',
           'B-NORP',
           'E-NORP',
           'S-QUANTITY',
           'B-PERCENT',
           'I-PERCENT',
           'E-PERCENT',
           'I-CARDINAL',
           'B-ORDINAL',
           'I-ORDINAL',
           'E-ORDINAL']
