# The labels for the output of the dp model.
# A string label can be obtained by an output index
dp_labels = \
['obl',
 'obj',
 'dep',
 'mark',
 'case',
 'flat',
 'nummod',
 'obl:arg',
 'punct',
 'cop',
 'acl:relcl',
 'expl',
 'nsubj',
 'csubj:pass',
 'root',
 'advmod',
 'nsubj:pass',
 'ccomp',
 'conj',
 'amod',
 'xcomp',
 'aux',
 'appos',
 'csubj',
 'fixed',
 'nmod',
 'iobj',
 'parataxis',
 'orphan',
 'det',
 'advcl',
 'vocative',
 'compound',
 'cc',
 'discourse',
 'acl',
 'obl:agent']